--------------------------------------------------------------------------------
-- UPC - EETAC - CSD - digsys.upc.edu     
--------------------------------------------------------------------------------
-- A standard circuit Dec_4_16: Binary decoder 4 to 16 (DEC4:16)
-- Plan B: behavioural approach. This is the translation of the schematic 
-- capturing the complete truth table in this page (Fig. 3) with the 
-- outputs active-low
-- https://digsys.upc.edu/csd/P02/DEC/Dec_4_16B/Dec_4_16.html

-- Project P2: standard logic circuits in VHDL
-- http://digsys.upc.edu
--------------------------------------------------------------------------------
LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

ENTITY Dec_4_16 IS
		PORT (
			D	: IN 	STD_LOGIC_VECTOR(3 DOWNTO 0);
			E	: IN  	STD_LOGIC;
			Y_L	: OUT  	STD_LOGIC_VECTOR(15 DOWNTO 0)    
			);
END Dec_4_16;

ARCHITECTURE truth_table OF Dec_4_16 IS

-- This signal is for grouping all the inputs into a single input vector cable:
signal X_in: STD_LOGIC_VECTOR (4 downto 0);

BEGIN
	PROCESS (X_in)  
		BEGIN
			CASE X_in IS
	       -- E D(3) D(2) D(1) D(0)    	Y_L(15..0)        
          	   -- -------------       -----------------    
				WHEN "10000" =>       		-- decoding 0
								Y_L <= "1111111111111110";
				WHEN "10001" =>       		-- decoding 1
								Y_L <= "1111111111111101";
				WHEN "10010" =>       		-- decoding 2
								Y_L <= "1111111111111011";						
				WHEN "10011" =>       		-- decoding 3
								Y_L <= "1111111111110111";
				WHEN "10100" =>       		-- 
								Y_L <= "1111111111101111";
				WHEN "10101" =>       		-- 
								Y_L <= "1111111111011111";
				WHEN "10110" =>       		-- 
								Y_L <= "1111111110111111";
				WHEN "10111" =>       		-- 
								Y_L <= "1111111101111111";
				WHEN "11000" =>       		-- 
								Y_L <= "1111111011111111";
				WHEN "11001" =>       		-- 
								Y_L <= "1111110111111111";
				WHEN "11010" =>       		-- 
								Y_L <= "1111101111111111";
				WHEN "11011" =>       		-- 
								Y_L <= "1111011111111111";
				WHEN "11100" =>       		-- 
								Y_L <= "1110111111111111";
				WHEN "11101" =>       		-- 
								Y_L <= "1101111111111111";
				WHEN "11110" =>       		-- decoding 14
								Y_L <= "1011111111111111";
				WHEN "11111" =>       		-- decoding 15
								Y_L <= "0111111111111111";
				WHEN Others =>       		-- disable the chip
								Y_L <= "1111111111111111"; 
			END CASE;
			
	END PROCESS;
		
-- Extra logic is required to connect the input buffers 
-- that adapt the inputs to the truth table artefact

X_in(4) <= E;    

X_in(3 downto 0) <= D;
 		   	   
END truth_table;
